package org.jenkinsci.plugins.p4_client.client;

import hudson.model.FreeStyleBuild;
import hudson.model.Result;
import hudson.model.Cause;
import hudson.model.CauseAction;
import hudson.model.FreeStyleProject;
import hudson.model.ParametersAction;

import java.io.File;
import java.io.IOException;
import java.util.logging.Logger;

import jenkins.model.Jenkins;
import jenkins.util.TimeDuration;

import org.jenkinsci.plugins.p4_client.P4Server;
import org.jenkinsci.plugins.p4_client.PerforceScm;
import org.jenkinsci.plugins.p4_client.credentials.P4PasswordImpl;
import org.jenkinsci.plugins.p4_client.workspace.StaticWorkspaceImpl;
import org.jenkinsci.plugins.p4_client.workspace.Workspace;
import org.junit.Test;
import org.jvnet.hudson.test.HudsonTestCase;

import com.cloudbees.plugins.credentials.CredentialsScope;
import com.cloudbees.plugins.credentials.SystemCredentialsProvider;

public class ConnectionTest extends HudsonTestCase {

	private static Logger logger = Logger.getLogger(ConnectionTest.class
			.getName());

	private final String P4ROOT = "tmp-p4root";
	private final String P4PORT = "localhost:1999";

	private final P4Server p4d = new P4Server("p4d", P4ROOT, P4PORT);

	@Test
	public void testCheckP4d() {
		try {
			int ver = p4d.getVersion();
			assertTrue(ver >= 20121);
		} catch (Exception e) {
			fail("Perforce Server P4D was not found");
		}
	}

	@Test
	public void testSampleDepotExtraction() {
		File ckp = new File("src/test/resources/checkpoint.gz");
		assertTrue(ckp.exists());

		File depot = new File("src/test/resources/depot.tar.gz");
		assertTrue(depot.exists());

		try {
			p4d.clean();
		} catch (IOException e) {
			fail("P4ROOT still exists, please clean up. " + P4ROOT);
		}
		File root = new File(P4ROOT);
		assertTrue(root.exists());

		try {
			p4d.restore(ckp);
		} catch (Exception e) {
			fail("P4D failed to restore checkpoint");
		}

		try {
			p4d.upgrade();
		} catch (Exception e) {
			fail("P4D failed to upgrade server");
		}

		try {
			p4d.extract(depot);
		} catch (Exception e) {
			fail("P4D failed to extract depot: " + depot.getName());
		}
	}

	@Test
	public void testStartStopServer() {
		try {
			p4d.start();
		} catch (Exception e) {
			fail("P4D failed to start server");
		} finally {
			try {
				p4d.stop();
			} catch (Exception e) {
				fail("P4D failed to stop server");
			}
		}
	}

	@Test
	public void testFreeStyleProject() throws Exception {
		try {
			p4d.start();

			FreeStyleProject project = createFreeStyleProject("testFreeStyleProject");

			P4PasswordImpl auth = new P4PasswordImpl(CredentialsScope.SYSTEM,
					"id", "desc", P4PORT, null, "jenkins", "jenkins");

			SystemCredentialsProvider.getInstance().getCredentials().add(auth);
			SystemCredentialsProvider.getInstance().save();

			Workspace workspace = new StaticWorkspaceImpl("none", "test.ws");
			PerforceScm scm = new PerforceScm("id", workspace);

			project.setScm(scm);
			project.save();
			
			FreeStyleBuild build = project.scheduleBuild2(0, new Cause.UserIdCause()).get();
			assertBuildStatus(Result.SUCCESS,build);
			
		} catch (Exception e) {
			fail("Failed to run FreeStyleProject: " + e.getCause());
			e.printStackTrace();
		} finally {
			try {
				p4d.stop();
			} catch (Exception e) {
				fail("P4D failed to stop server");
			}
		}
	}
}
